// Updated Script: Generate Postnet Barcodes with a 12th Checksum Digit
function generatePostnetBarcodes() {
    // Ensure all elements are placed in the "Print Data" layer
    var doc = app.activeDocument;
    var printDataLayer = getOrCreateLayer("Print Data");

    // Define barcode parameters
    var tallBarHeight = 9.53; // Tall bar height in mm
    var shortBarHeight = 3.81; // Short bar height in mm
    var barWidth = 1.52; // Fixed width of each bar in mm
    var totalWidth = 212; // Total barcode width in mm
    var digitCount = 10; // Number of random digits before the 11th digit

    // Postnet barcode mapping
    var postnetMapping = {
        '0': [1, 1, 0, 0, 0],
        '1': [0, 0, 0, 1, 1],
        '2': [0, 0, 1, 0, 1],
        '3': [0, 0, 1, 1, 0],
        '4': [0, 1, 0, 0, 1],
        '5': [0, 1, 0, 1, 0],
        '6': [0, 1, 1, 0, 0],
        '7': [1, 0, 0, 0, 1],
        '8': [1, 0, 0, 1, 0],
        '9': [1, 0, 1, 0, 0]
    };

    // Generate 10 random digits (shared for both barcodes)
    var baseDigits = '';
    for (var i = 0; i < digitCount; i++) {
        baseDigits += Math.floor(Math.random() * 10).toString();
    }

    // For barcodef, add '0' as the 11th digit
    var barcodefBase = baseDigits + '0';

    // For barcodeb, add '5' as the 11th digit
    var barcodebBase = baseDigits + '5';

    // Calculate the checksum (12th digit) for barcodef
    var barcodefChecksum = calculateChecksum(barcodefBase);

    // Calculate the checksum (12th digit) for barcodeb
    var barcodebChecksum = calculateChecksum(barcodebBase);

    // Combine the base and checksum to form complete barcodes
    var barcodef = barcodefBase + barcodefChecksum.toString();
    var barcodeb = barcodebBase + barcodebChecksum.toString();

    // Generate barcodes
    generateBarcode(barcodef, "barcodef", "bottom");
    generateBarcode(barcodeb, "barcodeb", "top");

    // Add regmarks
    addRegmarkF();
    addRegmarkB();

    function calculateChecksum(digits) {
        var sum = 0;
        for (var i = 0; i < digits.length; i++) {
            sum += parseInt(digits[i], 10);
        }
        var remainder = sum % 10;
        return remainder === 0 ? 0 : 10 - remainder;
    }

    function generateBarcode(barcodeDigits, groupName, position) {
        var totalBars = (barcodeDigits.length * 5) + 2;
        var totalBarWidth = barWidth * totalBars;
        var totalSpacing = totalWidth - totalBarWidth;
        var spacingBetweenBars = totalSpacing / (totalBars - 1);

        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var mmToPoints = 2.83465;
        var inchToPoints = 72;

        var cmykColor = new CMYKColor();
        cmykColor.cyan = 100;
        cmykColor.magenta = 100;
        cmykColor.yellow = 100;
        cmykColor.black = 100;

        var barcodeStartX, barcodeStartY, textStartX, textStartY;

        if (groupName === "barcodef") {
            var regmarkHeight = 3 * mmToPoints;
            var regmarkWidth = (artboardBounds[2] - artboardBounds[0]) - inchToPoints;
            var regmarkX = artboardBounds[0] + inchToPoints / 2;
            var regmarkY = artboardBounds[3];

            barcodeStartY = regmarkY + regmarkHeight;
            barcodeStartX = regmarkX + regmarkWidth - totalWidth * mmToPoints;

            var artboardCenterX = (artboardBounds[2] + artboardBounds[0]) / 2;
            textStartX = artboardCenterX;
            textStartY = regmarkY + regmarkHeight + (7 * mmToPoints);

            var textGroup = printDataLayer.groupItems.add();
            var textItem = textGroup.textFrames.add();
            textItem.contents = barcodeDigits.slice(0, 11); // Only display first 11 digits
            textItem.left = textStartX - (textItem.width / 2);
            textItem.top = textStartY;
            textItem.textRange.characterAttributes.size = 15;
            textGroup.name = "textboxf";
        } else if (groupName === "barcodeb") {
            barcodeStartX = artboardBounds[0];
            barcodeStartY = artboardBounds[1] - (3 * mmToPoints);

            addTextboxB(barcodeDigits.slice(0, 11)); // Only display first 11 digits
        }

        var group = printDataLayer.groupItems.add();
        group.name = groupName;

        function drawBar(group, x, y, height, width) {
            var rect = group.pathItems.rectangle(y, x, width, -height);
            rect.filled = true;
            rect.fillColor = cmykColor;
            rect.stroked = false;
        }

        var startX = barcodeStartX;
        var startY = barcodeStartY;

        drawBar(group, startX, startY, tallBarHeight * mmToPoints, barWidth * mmToPoints);
        startX += (barWidth + spacingBetweenBars) * mmToPoints;

        for (var digitIndex = 0; digitIndex < barcodeDigits.length; digitIndex++) {
            var digit = barcodeDigits[digitIndex];
            var bars = postnetMapping[digit];

            for (var j = 0; j < bars.length; j++) {
                var height = (bars[j] ? tallBarHeight : shortBarHeight) * mmToPoints;
                drawBar(group, startX, startY, height, barWidth * mmToPoints);
                startX += (barWidth + spacingBetweenBars) * mmToPoints;
            }
        }

        drawBar(group, startX, startY, tallBarHeight * mmToPoints, barWidth * mmToPoints);

        if (groupName === "barcodeb") {
            group.rotate(180, true, true, true, true, Transformation.BOTTOMLEFT);
            group.left = artboardBounds[0] + (0.5 * inchToPoints);
        }
    }

    function addTextboxB(barcodeValue) {
        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var inchToPoints = 72;

        var textX = (artboardBounds[0] + artboardBounds[2]) / 2;
        var textY = artboardBounds[1] - (0.25 * inchToPoints);

        var textGroup = printDataLayer.groupItems.add();
        var textItem = textGroup.textFrames.add();
        textItem.contents = barcodeValue.slice(0, 11); // Only display first 11 digits
        textItem.textRange.characterAttributes.size = 15;
        textItem.left = textX - (textItem.width / 2);
        textItem.top = textY;

        textItem.rotate(180);
        textGroup.name = "textboxb";
    }

    function addRegmarkF() {
        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var mmToPoints = 2.83465;
        var inchToPoints = 72;

        var cmykColor = new CMYKColor();
        cmykColor.cyan = 100;
        cmykColor.magenta = 100;
        cmykColor.yellow = 100;
        cmykColor.black = 100;

        var regmarkHeight = 3 * mmToPoints;
        var regmarkWidth = (artboardBounds[2] - artboardBounds[0]) - inchToPoints;
        var regmarkX = artboardBounds[0] + inchToPoints / 2;
        var regmarkY = artboardBounds[3];

        var regmarkGroup = printDataLayer.groupItems.add();
        regmarkGroup.name = "regmarkf";

        var regmark = regmarkGroup.pathItems.rectangle(regmarkY, regmarkX, regmarkWidth, -regmarkHeight);
        regmark.filled = true;
        regmark.fillColor = cmykColor;
        regmark.stroked = false;
    }

    function addRegmarkB() {
        var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
        var artboardBounds = artboard.artboardRect;

        var mmToPoints = 2.83465;
        var inchToPoints = 72;

        var cmykColor = new CMYKColor();
        cmykColor.cyan = 100;
        cmykColor.magenta = 100;
        cmykColor.yellow = 100;
        cmykColor.black = 100;

        var regmarkHeight = 3 * mmToPoints;
        var regmarkWidth = (artboardBounds[2] - artboardBounds[0]) - inchToPoints;
        var regmarkX = artboardBounds[0] + inchToPoints / 2;
        var regmarkY = artboardBounds[1] - regmarkHeight;

        var regmarkGroup = printDataLayer.groupItems.add();
        regmarkGroup.name = "regmarkb";

        var regmark = regmarkGroup.pathItems.rectangle(regmarkY, regmarkX, regmarkWidth, -regmarkHeight);
        regmark.filled = true;
        regmark.fillColor = cmykColor;
        regmark.stroked = false;
    }

    function getOrCreateLayer(layerName) {
        for (var i = 0; i < doc.layers.length; i++) {
            if (doc.layers[i].name === layerName) {
                return doc.layers[i];
            }
        }
        var newLayer = doc.layers.add();
        newLayer.name = layerName;
        return newLayer;
    }
}

// Call the second script
generatePostnetBarcodes();